/***************************************************************************
 *  LX - Logs Xchecker
 *
 *  Copyright (c)2002 - 2004 Andrej  OM1CW
 *  Copyright (c)2005 Mike Karas OM4AA <mike.om4aa@gmail.com>
 *  Copyright (c)2009 Ladislav Vaiz OK1ZIA <ok1zia@nagano.cz>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include "osdep.h"
#include "debug.h"
#include "main.h"
#include "convert.h"
#include "qrb.h"
#include "check.h"
#include "compare.h"
#include "load.h"
#include "stats.h"
#include "cdwdb.h"
#include "claimed.h"
#include "top.h"
#include "diplom.h"
#include <string.h>


uint8_t *buf_ptr, *buf_free;
uint32_t buf_size = 0x10000;
uint32_t mem_used = 0x10000;
uint32_t calls = 0;
uint32_t all_entries = 0;

char ctest_name[BUFF_SIZE + 1] = "";
char cfg_epath[BUFF_SIZE + 1] = "ERR";

FILE *err, *inf;
char eprintf_load = 0;
char *eprintf_filename;

char space[20] = "                   ";

struct CALL_INFO *first_call[BAND_NUM], *last_call[BAND_NUM];

int verbose_mode = 3;
int forcedwwl_num = 0;
struct forced_wwls forced[1024];
int bad_calls_cnt = 0;
struct bad_calls_t bad_calls[1024];
//int no_slashes = 0;
int fatals=0;


int eprintf(const char *format, ...)
{
    DEBUG_FUNC("eprintf");

    va_list argptr;
    va_start(argptr, format);

    if (!eprintf_load)
        vprintf(format, argptr);
    else
        printf("!");
/*  if (eprintf_load) {
        fprintf(err, "\n+ Loading: %s - ", eprintf_filename);
        eprintf_load = 0;
    }*/

/*  vfprintf(inf, format, argptr);*/
    return vfprintf(err, format, argptr);
}

int main(int argc, char *argv[])
{
    int band, i, num;
    char text[BUFF_SIZE + 1];
    char textlp[BUFF_SIZE + 1];
    char textqrp[BUFF_SIZE + 1];
    FILE *fd_results;
    struct CALL_INFO *c;
    char lpstr[20];
    char qrpstr[20];

    printf("+ Logs Xchecker v"PACKAGE_VERSION" by OM1CW, OM4AA, OK1ZIA\n");

    memset(bandcat2lp, 0, sizeof(bandcat2lp));
    memset(bandcat2qrp, 0, sizeof(bandcat2qrp));

    for (band = 0; band < BAND_NUM; band++) {
        first_call[band] = NULL;
        last_call[band] = NULL;
    }

    buf_ptr = (uint8_t *) malloc(buf_size);
    buf_free = buf_ptr;

    if (!buf_ptr) {
        printf("! Not enought memory\n");
        return -1;
    }

    if (argc >= 2) {
        load_cfg(argv[1]);
    } else
        load_cfg(CONFIG_FILE);


/*    for (i=0; i< num_categs; i++){
        printf("%d. '%s' %d '%s'\n", i, categs[i].psect, categs[i].band, categs[i].name);
    }*/

    if (num_categs == 1) {
        printf("! Cfg file error\n");
        return -1;
    }

    eprintf_load = 0;

    printf("+ Progress bar [0123456789ABCDEFGHIJKLM]\n");

    for (band = 1; band < BAND_NUM; band++) {
        if (first_call[band]) {
            if (band >= BAND_GHZ) {
                if (band >= BAND_GHZ_ZERO)
                    sprintf(text, "%2.0f GHz", band2freq[band]);
                else
                    sprintf(text, "%2.1f GHz", band2freq[band]);
            } else
                sprintf(text, "%2.0f MHz", band2freq[band]);

            text[8] = 0;
            printf("+ Band: %s%s", text,
                   space + strlen(space) + strlen(text) - 8);

            make_bad_call(band);

            make_refs(band);

            make_call_cmp(band);

            make_refs(band);
            make_loc_cmp(band);
            
            make_valid_rst(band);
            make_dump();
            make_xchk2(band, 1, 127, "all");
            make_xchk2(band, 2, 127 - 32 - 16, "all but RST");

            make_xchk2(band, 3, 127 - 8, "all but NRr");
            make_xchk2(band, 3, 127 - 4, "all but NRs");
            make_xchk2(band, 3, 127 - 8 - 4, "all but NR");

            make_xchk2(band, 4, 127 - 2, "all but WWLr");
            make_xchk2(band, 4, 127 - 1, "all but WWLs");
            make_xchk2(band, 4, 127 - 2 - 1, "all but WWL");
            
            make_xchk2(band, 5, 127 - 64,    "all but CALL");
            make_xchk2(band, 6, 127 - 32 - 16 - 8 - 4, "CALL+WWL");
            make_xchk2(band, 7, 127 - 32 - 16 - 2 - 1, "CALL+NR");
            make_xchk2(band, 8, 127 - 64 - 32 - 16, "NR+WWL");
            //make_xchk2(band, 9, 127 - 64 - 8 - 4, "RST+WWL");
            //make_xchk2(band, 10, 127 - 64 - 2 - 1, "RST+NR");
            make_xchk2(band, 11, 127 - 8 - 4 - 2 - 1, "CALL+RST");
            make_xchk2(band, 12, 64, "CALL");
            //make_xchk2(band, 13, 2 + 1, "WWL");
            make_xchk_res(band); /* fill in ok_rst ok_nr ok_loc */

            make_not_in_log(band);
            make_dupe(band);

            eprintf_load = 1;
            make_outside_wwl(band);
            eprintf_load = 0;


            make_band_refs(band);
            make_refs(band);
            make_err_files(band, text);

            printf("\n");
        }
    }

#ifdef HAVE_GLIB2
    if (cw) /* free used CW db */
        free_cw(cw);
    if (dw) /* free used DW and CTY db */
        free_dw(dw);
#endif

    FILE *htm, *res, *dip, *claim_htm, *claim_txt;

    sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, CLAIM_HTML_FILE);
    claim_htm = fopen(text, "wt");
    if (!claim_htm) {
        printf("! Cannot create %s file\n", text);
        return -1;
    }

    sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, CLAIM_TXT_FILE);
    claim_txt = fopen(text, "wt");
    if (!claim_txt) {
        printf("! Cannot create %s file\n", text);
        return -1;
    }

    sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, HTML_FILE);
    htm = fopen(text, "wt");
    if (!htm) {
        printf("! Cannot create %s file\n", text);
        return -1;
    }

    sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, REFERENCES_FILE);
    res = fopen(text, "wt");
    if (!res) {
        printf("! Cannot create %s file\n", text);
        return -1;
    }

    if (diplompath && *diplompath){
        sprintf(text, "%s%s%s", diplompath, DIR_DELIMITER, "index.html");
        dip = fopen(text, "wt");
        if (!dip) {
            printf("! Cannot create %s file\n", text);
            return -1;
        }
    }

    sprintf(text, "%s%s%s", cfg_epath, DIR_DELIMITER, RESULTS_FILE);
    if (!(fd_results = fopen(text, "wt"))) {
        printf("! Cannot create %s file\n", text);
        return -1;
    }
    fprintf(claim_txt, 
            "Claimed score of contest: %s\n", ctest_name);


    fprintf(claim_htm,
            "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
    fprintf(claim_htm,
            "<html>\n"
            "<head>\n"
            "<style type=\"text/css\">\n"
            "body {\n"
            "    background: #f8f8f8;\n"
            "    color: #000000;\n"
            "    font-size: 12pt;\n"
            "    font-family: Tahoma, Helvetica, Arial, Arial CE, sans-serif;\n"
            "}\n"
            "table.sortable {\n"
            "    border: black 2px solid;\n"
            "    background: #f0f0f0;\n"
            "}\n"
            "table.sortable thead {\n"
            "    color: blue;\n"
            "    background: #d0d0d0;\n"
            "    cursor: pointer;\n"
            "}\n"
            "table.sortable td{\n"
            "   padding: 2px\n"
            "   padding-left: 4px\n"    
            "   padding-right: 4px\n"    
            "}\n"
            "</style>\n"
            "<title>Claimed score of contest: %s</title>\n"
            "<script type=\"text/javascript\" src=\"sorttable.js\"></script>\n"
            "<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-2\">\n"
            "</head>\n"
            "<body>\n",
            ctest_name);
    fprintf(claim_htm,
            "<h1>Claimed score of contest: %s</h1>\n",
            ctest_name);

    fprintf(htm,
            "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
    fprintf(htm,
            "<html>\n"
            "<head>\n"
            "<style type=\"text/css\">\n"
            "body {\n"
            "    background: #f8f8f8;\n"
            "    color: #000000;\n"
            "    font-size: 12pt;\n"
            "    font-family: Tahoma, Helvetica, Arial, Arial CE, sans-serif;\n"
            "}\n"
            "table.sortable {\n"
            "    border: black 2px solid;\n"
            "    background: #f0f0f0;\n"
            "}\n"
            "table.sortable thead {\n"
            "    color: blue;\n"
            "    background: #d0d0d0;\n"
            "    cursor: pointer;\n"
            "}\n"
            "table.sortable td{\n"
            "   padding-left: 4px;\n"    
            "   padding-right: 4px;\n"    
            "}\n"
            "</style>\n"
            "<title>Result of contest: %s</title>\n"
            "<script type=\"text/javascript\" src=\"sorttable.js\"></script>\n"
            "<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-2\">"
            "</head>\n"
            "<body>\n",
            ctest_name);
    // fprintf(htm, "<html>\n<body>\n");
    fprintf(htm,
            "<h1>Result of contest: %s</h1>\n",
            ctest_name);
    fprintf(res, "\nContest: %s\n\n", ctest_name);
    
    if (dip){
        fprintf(dip,
                "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\">\n");
        fprintf(dip,
                "<html>\n"
                "<head>\n"
                "<style type=\"text/css\">\n"
                "body {\n"
                "    background: #f8f8f8;\n"
                "    color: #000000;\n"
                "    font-size: 12pt;\n"
                "    font-family: Tahoma, Helvetica, Arial, Arial CE, sans-serif;\n"
                "}\n"
                "table.sortable {\n"
                "    border: black 2px solid;\n"
                "    background: #f0f0f0;\n"
                "}\n"
                "table.sortable thead {\n"
                "    color: blue;\n"
                "    background: #d0d0d0;\n"
                "    cursor: pointer;\n"
                "}\n"
                "table.sortable td{\n"
                "   padding-left: 4px;\n"    
                "   padding-right: 4px;\n"    
                "}\n"
                "</style>\n"
                "<title>Result of contest: %s</title>\n"
                "<script type=\"text/javascript\" src=\"sorttable.js\"></script>\n"
                "<meta http-equiv=\"content-type\" content=\"text/html; charset=iso-8859-2\">"
                "</head>\n"
                "<body>\n",
                ctest_name);
        fprintf(dip,
                "<h1>Awards of: %s</h1>\n",
                ctest_name);
    }
    
    /* XML Results File Header */
    fprintf(fd_results, "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n");
    fprintf(fd_results, "<LXResults contest=\"%s\">\n", ctest_name);

    fprintf(claim_htm, "<ul>\n");
    fprintf(htm, "<ul>\n");
    for (band = 1; band < BAND_NUM; band++) {
        if (first_call[band]) {
            if (band >= BAND_GHZ) {
                if (band >= BAND_GHZ_ZERO)
                    sprintf(text, "%2.0f GHz", band2freq[band]);
                else
                    sprintf(text, "%2.1f GHz", band2freq[band]);
            } else
                sprintf(text, "%2.0f MHz", band2freq[band]);
            text[8] = 0;
            sprintf(textlp, "%s LP", text); 
            sprintf(textqrp, "%s QRP", text); 

            for (i = 1; i < num_categs; i++) {
                int lp = band2lp[band]; 
                if (bandcat2lp[band][i] > 0) lp = bandcat2lp[band][i];

                int qrp = band2qrp[band];
                if (bandcat2qrp[band][i] > 0) qrp = bandcat2qrp[band][i];
                
                num = 0;
                for (c = first_call[band]; c; c = c->next){
                    if (!inclcall(c->pwr, lp, qrp, 0)) continue;
                    if (c->categ == i) num++;
                }
                if (num){
                    fprintf(claim_htm, "<li><a href=\"#%d_%d\">%s, %s</a></li>\n", band, i, text, categs[i].name);
                    fprintf(htm, "<li><a href=\"#%d_%d\">%s, %s</a></li>\n", band, i, text, categs[i].name);
                }

                num = 0;
                for (c = first_call[band]; c; c = c->next){
                    if (!inclcall(c->pwr, lp, qrp, 1)) continue;
                    if (c->categ == i) num++;
                }                     
                if (num){
                    sprintf(lpstr, "_%d", lp);
                    fprintf(claim_htm, "<li><a href=\"#%d_%d%s\">%s, %s</a></li>\n", band, i, lpstr, textlp, categs[i].name); 
                    fprintf(htm, "<li><a href=\"#%d_%d%s\">%s, %s</a></li>\n", band, i, lpstr, textlp, categs[i].name);
                }

                num = 0;
                for (c = first_call[band]; c; c = c->next){
                    if (!inclcall(c->pwr, lp, qrp, 2)) continue;
                    if (c->categ == i) num++;
                }                     
                if (num){
                    sprintf(lpstr, "_%d", qrp);
                    fprintf(claim_htm, "<li><a href=\"#%d_%d%s\">%s, %s</a></li>\n", band, i, qrpstr, textqrp, categs[i].name); 
                    fprintf(htm, "<li><a href=\"#%d_%d%s\">%s, %s</a></li>\n", band, i, qrpstr, textqrp, categs[i].name);
                }

            }
        }

    }
    fprintf(claim_htm, "</ul>\n");
    fprintf(htm, "</ul>\n");

    for (band = 1; band < BAND_NUM; band++) {
        if (first_call[band]) {
            if (band >= BAND_GHZ) {
                if (band >= BAND_GHZ_ZERO)
                    sprintf(text, "%2.0f GHz", band2freq[band]);
                else
                    sprintf(text, "%2.1f GHz", band2freq[band]);
            } else
                sprintf(text, "%2.0f MHz", band2freq[band]);
            text[8] = 0;
            
            g_do_check = 0;
            sprintf(textlp, "%s LP", text);
            sprintf(textqrp, "%s QRP", text);
            for (i = 1; i < num_categs; i++) {
                int lp = band2lp[band];
                if (bandcat2lp[band][i] > 0) lp = bandcat2lp[band][i];
                
                int qrp = band2qrp[band];
                if (bandcat2qrp[band][i] > 0) qrp = bandcat2qrp[band][i];

                make_claimed(   band, text, i,      claim_htm, claim_txt,  lp, qrp, 0);
                make_score_list(band, text, i, res, htm,       fd_results, dip, lp, qrp, 0);
                if (lp > 0){
                    make_claimed(   band, textlp, i,      claim_htm, claim_txt,  lp, qrp, 1);
                    make_score_list(band, textlp, i, res, htm,       fd_results, dip, lp, qrp, 1);
                }
                if (qrp > 0){
                    make_claimed(   band, textqrp, i,      claim_htm, claim_txt,  lp, qrp, 2);
                    make_score_list(band, textqrp, i, res, htm,       fd_results, dip, lp, qrp, 2);
                }
            }

         	make_top(band, htm, res);
            if (g_do_check) make_check_list(band, res, htm);
        }
    }

    fprintf(claim_txt, "\nProcessed by %s %s\n", PACKAGE_NAME, PACKAGE_VERSION);

    fprintf(claim_htm, "<p>Processed by <a href=\"http://tucnak.nagano.cz/wiki/Lx\">%s %s</a></p>\n", PACKAGE_NAME, PACKAGE_VERSION);
    fprintf(claim_htm, "</body>\n</html>\n");
    
    fprintf(htm, "<p>Processed by <a href=\"http://tucnak.nagano.cz/wiki/Lx\">%s %s</a></p>\n", PACKAGE_NAME, PACKAGE_VERSION);
    fprintf(htm, "</body>\n</html>\n");
    
    if (dip){
        fprintf(dip, "<p>Processed by <a href=\"http://tucnak.nagano.cz/wiki/Lx\">%s %s</a></p>\n", PACKAGE_NAME, PACKAGE_VERSION);
        fprintf(dip, "</body>\n</html>\n");
    }
    
    /* XML Results File Footer */
    fprintf(fd_results, "</LXResults>\n");


    make_uniq();
 //   if (verbose_mode >= 4) make_dump();

    fclose(fd_results);
    fclose(inf);
    fclose(res);
    fclose(htm);
    if (dip) fclose(dip);
    fclose(claim_htm);
    fclose(claim_txt);

    fclose(err);

    return 0;
}

